@@ -1,6 +1,9 @@
 use std::sync::Arc;
 
+#[cfg(feature = "streaming")]
+use crate::spirc;
 use crate::{
+    config,
     event::{ClientRequest, PlayerRequest},
     state::*,
 };
@@ -12,6 +15,7 @@ mod handlers;
 mod spotify;
 
 pub use handlers::*;
+use tokio::sync::{broadcast, mpsc};
 
 /// The application's client
 #[derive(Clone)]
@@ -22,13 +26,38 @@ pub struct Client {
 
 impl Client {
     /// creates a new client
-    pub fn new(session: Session, client_id: String) -> Self {
+    pub fn new(session: Session, device: config::DeviceConfig, client_id: String) -> Self {
         Self {
-            spotify: Arc::new(spotify::Spotify::new(session, client_id)),
+            spotify: Arc::new(spotify::Spotify::new(session, device, client_id)),
             http: reqwest::Client::new(),
         }
     }
 
+    /// creates a new Librespot's spirc connection
+    #[cfg(feature = "streaming")]
+    pub async fn new_spirc_connection(
+        &self,
+        spirc_sub: broadcast::Receiver<()>,
+        client_pub: mpsc::Sender<ClientRequest>,
+        should_connect: bool,
+    ) -> Result<()> {
+        let session = match self.spotify.session {
+            None => return Ok(()),
+            Some(ref session) => session.clone(),
+        };
+        let device = self.spotify.device.clone();
+        let device_id = session.device_id().to_string();
+        spirc::new_connection(session, device, client_pub, spirc_sub);
+
+        // whether should we connect to the new spirc client upon its creation
+        if should_connect {
+            tracing::info!("transfer playback to the new spirc client with id = {device_id}");
+            self.spotify.transfer_playback(&device_id, None).await?;
+        }
+
+        Ok(())
+    }
+
     /// initializes the authorization token inside the Spotify client
     pub async fn init_token(&self) -> Result<()> {
         self.spotify.refresh_token().await?;
@@ -47,7 +76,7 @@ impl Client {
     ) -> Result<()> {
         tracing::info!("handle player request: {:?}", request);
 
-        // `TransferPlayback` and `Reconnect` need to be handled separately
+        // `TransferPlayback` needs to be handled separately
         // from other play requests because they don't require an active playback
 
         // transfer the current playback to another device
@@ -59,36 +88,6 @@ impl Client {
             tracing::info!("transfered the playback to device with {} id", device_id);
             return Ok(());
         }
-        // trying to reconnect to the first available device
-        if let PlayerRequest::Reconnect = request {
-            let device_id = self.spotify.device().await?.into_iter().find_map(|d| d.id);
-
-            match device_id {
-                Some(id) => {
-                    tracing::info!(
-                        "transfered the playback to the first available device (id={})",
-                        id
-                    );
-                    self.spotify.transfer_playback(&id, None).await?;
-                }
-                None => {
-                    // if the streaming is available and no device is found,
-                    // which is probably because user doesn't specify their own client ID,
-                    // try to connect to the integrated client's device ID
-                    #[cfg(feature = "streaming")]
-                    {
-                        let device_id = self.spotify.session.session()?.device_id();
-                        self.spotify.transfer_playback(device_id, None).await?;
-                        tracing::info!(
-                            "transfered the playback to the integrated client's device (id={})",
-                            device_id
-                        );
-                    }
-                }
-            }
-
-            return Ok(());
-        }
 
         let playback = match state.player.read().simplified_playback() {
             Some(playback) => playback,
@@ -137,7 +136,6 @@ impl Client {
                     .await?
             }
             PlayerRequest::TransferPlayback(..) => unreachable!(),
-            PlayerRequest::Reconnect => unreachable!(),
         };
 
         Ok(())
@@ -148,6 +146,10 @@ impl Client {
         tracing::info!("handle client request {:?}", request);
 
         match request {
+            #[cfg(feature = "streaming")]
+            ClientRequest::NewSpircConnection => {
+                unreachable!("request should be already handled by the caller function");
+            }
             ClientRequest::GetCurrentUser => {
                 let user = self.spotify.current_user().await?;
                 state.data.write().user_data.user = Some(user);
@@ -179,8 +181,7 @@ impl Client {
                 let devices = self.spotify.device().await?;
                 state.player.write().devices = devices
                     .into_iter()
-                    .map(Device::try_from_device)
-                    .flatten()
+                    .filter_map(Device::try_from_device)
                     .collect();
             }
             ClientRequest::GetUserPlaylists => {
@@ -195,6 +196,20 @@ impl Client {
                 let albums = self.current_user_saved_albums().await?;
                 state.data.write().user_data.saved_albums = albums;
             }
+            ClientRequest::GetUserTopTracks => {
+                let id = "top-tracks";
+                if !state.data.read().caches.tracks.contains(id) {
+                    let tracks = self.current_user_top_tracks().await?;
+                    state.data.write().caches.tracks.put(id.to_string(), tracks);
+                }
+            }
+            ClientRequest::GetUserRecentlyPlayedTracks => {
+                let id = "recently-played-tracks";
+                if !state.data.read().caches.tracks.contains(id) {
+                    let tracks = self.current_user_recently_played_tracks().await?;
+                    state.data.write().caches.tracks.put(id.to_string(), tracks);
+                }
+            }
             ClientRequest::GetContext(context) => {
                 let uri = context.uri();
                 if !state.data.read().caches.context.contains(&uri) {
@@ -217,11 +232,11 @@ impl Client {
                 }
             }
             ClientRequest::GetRecommendations(seed) => {
-                let uri = seed.uri();
-                if !state.data.read().caches.recommendation.contains(&uri) {
+                let id = format!("recommendations::{}", seed.uri());
+                if !state.data.read().caches.tracks.contains(&id) {
                     let tracks = self.recommendations(&seed).await?;
 
-                    state.data.write().caches.recommendation.put(uri, tracks);
+                    state.data.write().caches.tracks.put(id, tracks);
                 }
             }
             ClientRequest::AddTrackToPlaylist(playlist_id, track_id) => {
@@ -235,6 +250,73 @@ impl Client {
         Ok(())
     }
 
+    // connects to the first available device
+    pub async fn connect_to_first_available_device(&self) -> Result<()> {
+        let device_id = self.spotify.device().await?.into_iter().find_map(|d| d.id);
+
+        match device_id {
+            Some(id) => {
+                tracing::info!(
+                    "transfered the playback to the first available device (id={})",
+                    id
+                );
+                self.spotify.transfer_playback(&id, None).await?;
+            }
+            None => {
+                // if the streaming is available and no device is found,
+                // try to connect to the integrated client's device
+                #[cfg(feature = "streaming")]
+                {
+                    if let Some(ref session) = self.spotify.session {
+                        let device_id = session.device_id();
+                        self.spotify.transfer_playback(device_id, None).await?;
+                        tracing::info!(
+                            "transfered the playback to the integrated client's device (id={})",
+                            device_id
+                        );
+                    }
+                }
+            }
+        }
+
+        Ok(())
+    }
+
+    /// gets the recently played tracks of the current user
+    pub async fn current_user_recently_played_tracks(&self) -> Result<Vec<Track>> {
+        let first_page = self
+            .spotify
+            .current_user_recently_played(Some(50), None)
+            .await?;
+
+        let play_histories = self.all_cursor_based_paging_items(first_page).await?;
+
+        // de-duplicate the tracks returned from the recently-played API
+        let mut tracks = Vec::<Track>::new();
+        for history in play_histories {
+            if !tracks.iter().any(|t| t.name == history.track.name) {
+                if let Some(track) = Track::try_from_full_track(history.track) {
+                    tracks.push(track);
+                }
+            }
+        }
+        Ok(tracks)
+    }
+
+    /// gets the top tracks of the current user
+    pub async fn current_user_top_tracks(&self) -> Result<Vec<Track>> {
+        let first_page = self
+            .spotify
+            .current_user_top_tracks_manual(None, Some(50), None)
+            .await?;
+
+        let tracks = self.all_paging_items(first_page).await?;
+        Ok(tracks
+            .into_iter()
+            .filter_map(Track::try_from_full_track)
+            .collect())
+    }
+
     /// gets all playlists of the current user
     pub async fn current_user_playlists(&self) -> Result<Vec<Playlist>> {
         let first_page = self
@@ -316,8 +398,7 @@ impl Client {
         // converts `rspotify_model::SimplifiedAlbum` into `state::Album`
         let albums = albums
             .into_iter()
-            .map(Album::try_from_simplified_album)
-            .flatten()
+            .filter_map(Album::try_from_simplified_album)
             .collect();
         Ok(self.clean_up_artist_albums(albums))
     }
@@ -393,11 +474,22 @@ impl Client {
             }
         };
 
-        Ok(tracks
+        let mut tracks = tracks
             .into_iter()
-            .map(Track::try_from_simplified_track)
-            .flatten()
-            .collect())
+            .filter_map(Track::try_from_simplified_track)
+            .collect::<Vec<_>>();
+
+        // for track recommendation seed, add the track seed to the returned recommended tracks
+        if let SeedItem::Track(track) = seed {
+            let mut seed_track = track.clone();
+            // recommended tracks returned from the API are represented by `SimplifiedTrack` struct,
+            // which doesn't have `album` field specified. So, we need to change the seed track's
+            // `album` field for consistency with other tracks in the list.
+            seed_track.album = None;
+            tracks.insert(0, seed_track);
+        }
+
+        Ok(tracks)
     }
 
     /// searchs for items (tracks, artists, albums, playlists) that match a given query string.
@@ -414,8 +506,7 @@ impl Client {
                 rspotify_model::SearchResult::Tracks(p) => p
                     .items
                     .into_iter()
-                    .map(Track::try_from_full_track)
-                    .flatten()
+                    .filter_map(Track::try_from_full_track)
                     .collect(),
                 _ => unreachable!(),
             },
@@ -429,8 +520,7 @@ impl Client {
                 rspotify_model::SearchResult::Albums(p) => p
                     .items
                     .into_iter()
-                    .map(Album::try_from_simplified_album)
-                    .flatten()
+                    .filter_map(Album::try_from_simplified_album)
                     .collect(),
                 _ => unreachable!(),
             },
@@ -553,13 +643,12 @@ impl Client {
             .all_paging_items(first_page)
             .await?
             .into_iter()
-            .map(|item| match item.track {
+            .filter_map(|item| match item.track {
                 Some(rspotify_model::PlayableItem::Track(track)) => {
                     Track::try_from_full_track(track)
                 }
                 _ => None,
             })
-            .flatten()
             .collect::<Vec<_>>();
 
         Ok(Context::Playlist {
@@ -584,7 +673,7 @@ impl Client {
             .all_paging_items(first_page)
             .await?
             .into_iter()
-            .map(|t| {
+            .filter_map(|t| {
                 // simplified track doesn't have album so
                 // we need to manually include one during
                 // converting into `state::Track`
@@ -593,7 +682,6 @@ impl Client {
                     t
                 })
             })
-            .flatten()
             .collect::<Vec<_>>();
 
         Ok(Context::Album { album, tracks })
@@ -612,8 +700,7 @@ impl Client {
             .artist_top_tracks(artist_id, &rspotify_model::enums::misc::Market::FromToken)
             .await?
             .into_iter()
-            .map(Track::try_from_full_track)
-            .flatten()
+            .filter_map(Track::try_from_full_track)
             .collect::<Vec<_>>();
 
         let related_artists = self
@@ -669,8 +756,27 @@ impl Client {
         Ok(items)
     }
 
+    /// gets all cursor-based paging items starting from a pagination object of the first page
+    async fn all_cursor_based_paging_items<T>(
+        &self,
+        first_page: rspotify_model::CursorBasedPage<T>,
+    ) -> Result<Vec<T>>
+    where
+        T: serde::de::DeserializeOwned,
+    {
+        let mut items = first_page.items;
+        let mut maybe_next = first_page.next;
+        while let Some(url) = maybe_next {
+            tracing::info!("url: {url}");
+            let mut next_page = self.internal_call::<CursorBasedPage<T>>(&url).await?;
+            items.append(&mut next_page.items);
+            maybe_next = next_page.next;
+        }
+        Ok(items)
+    }
+
     /// updates the current playback state
-    async fn update_current_playback_state(&self, state: &SharedState) -> Result<()> {
+    pub async fn update_current_playback_state(&self, state: &SharedState) -> Result<()> {
         let playback = self.spotify.current_playback(None, None::<Vec<_>>).await?;
         let mut player = state.player.write();
         player.playback = playback;
