@@ -79,6 +79,10 @@ impl Default for KeymapConfig {
                     key_sequence: "a".into(),
                     command: Command::ShowActionsOnCurrentTrack,
                 },
+                Keymap {
+                    key_sequence: "R".into(),
+                    command: Command::ReconnectIntegratedClient,
+                },
                 Keymap {
                     key_sequence: "tab".into(),
                     command: Command::FocusNextWindow,
@@ -109,7 +113,15 @@ impl Default for KeymapConfig {
                 },
                 Keymap {
                     key_sequence: "g space".into(),
-                    command: Command::BrowsePlayingContext,
+                    command: Command::CurrentlyPlayingContextPage,
+                },
+                Keymap {
+                    key_sequence: "g t".into(),
+                    command: Command::TopTrackPage,
+                },
+                Keymap {
+                    key_sequence: "g r".into(),
+                    command: Command::RecentlyPlayedTrackPage,
                 },
                 Keymap {
                     key_sequence: "g l".into(),
