@@ -2,14 +2,12 @@ use crate::{
     command::Command,
     key::{Key, KeySequence},
     state::*,
-    utils::new_list_state,
+    utils::{new_list_state, new_table_state},
 };
 use anyhow::Result;
-use crossterm::event::*;
-use rand::Rng;
-use std::sync::mpsc;
-use tokio_stream::StreamExt;
+use tokio::sync::mpsc;
 
+mod page;
 mod popup;
 mod window;
 
@@ -23,7 +21,6 @@ pub enum PlayerRequest {
     Repeat,
     Shuffle,
     Volume(u8),
-    Reconnect,
     TransferPlayback(String, bool),
     StartPlayback(Playback),
 }
@@ -36,6 +33,8 @@ pub enum ClientRequest {
     GetUserPlaylists,
     GetUserSavedAlbums,
     GetUserFollowedArtists,
+    GetUserTopTracks,
+    GetUserRecentlyPlayedTracks,
     GetContext(ContextId),
     GetCurrentPlayback,
     GetRecommendations(SeedItem),
@@ -43,42 +42,35 @@ pub enum ClientRequest {
     AddTrackToPlaylist(PlaylistId, TrackId),
     SaveToLibrary(Item),
     Player(PlayerRequest),
+    #[cfg(feature = "streaming")]
+    NewSpircConnection,
 }
 
-#[tokio::main]
 /// starts a terminal event handler (key pressed, mouse clicked, etc)
-pub async fn start_event_handler(send: mpsc::Sender<ClientRequest>, state: SharedState) {
-    let mut event_stream = EventStream::new();
+pub fn start_event_handler(state: SharedState, client_pub: mpsc::Sender<ClientRequest>) {
+    while let Ok(event) = crossterm::event::read() {
+        tracing::info!("got a terminal event: {:?}", event);
 
-    while let Some(event) = event_stream.next().await {
-        match event {
-            Ok(event) => {
-                tracing::info!("got a terminal event: {:?}", event);
-
-                if let Err(err) = match event {
-                    Event::Mouse(event) => handle_mouse_event(event, &send, &state),
-                    Event::Key(event) => handle_key_event(event, &send, &state),
-                    _ => Ok(()),
-                } {
-                    tracing::warn!("failed to handle event: {}", err);
-                }
-            }
-            Err(err) => {
-                tracing::warn!("failed to get event: {}", err);
-            }
+        if let Err(err) = match event {
+            crossterm::event::Event::Mouse(event) => handle_mouse_event(event, &client_pub, &state),
+            crossterm::event::Event::Key(event) => handle_key_event(event, &client_pub, &state),
+            _ => Ok(()),
+        } {
+            tracing::warn!("failed to handle event: {}", err);
         }
     }
 }
 
 // handles a terminal mouse event
 fn handle_mouse_event(
-    event: MouseEvent,
-    send: &mpsc::Sender<ClientRequest>,
+    event: crossterm::event::MouseEvent,
+    client_pub: &mpsc::Sender<ClientRequest>,
     state: &SharedState,
 ) -> Result<()> {
     let ui = state.ui.lock();
     // a left click event
-    if let MouseEventKind::Down(MouseButton::Left) = event.kind {
+    if let crossterm::event::MouseEventKind::Down(crossterm::event::MouseButton::Left) = event.kind
+    {
         if event.row == ui.progress_bar_rect.y {
             // calculate the seek position (in ms) based on the clicked position,
             // the progress bar's width and the track's duration (in ms)
@@ -88,7 +80,8 @@ fn handle_mouse_event(
             if let Some(track) = track {
                 let position_ms = (track.duration.as_millis() as u32) * (event.column as u32)
                     / (ui.progress_bar_rect.width as u32);
-                send.send(ClientRequest::Player(PlayerRequest::SeekTrack(position_ms)))?;
+                client_pub
+                    .blocking_send(ClientRequest::Player(PlayerRequest::SeekTrack(position_ms)))?;
             }
         }
     }
@@ -97,8 +90,8 @@ fn handle_mouse_event(
 
 // handle a terminal key pressed event
 fn handle_key_event(
-    event: KeyEvent,
-    send: &mpsc::Sender<ClientRequest>,
+    event: crossterm::event::KeyEvent,
+    client_pub: &mpsc::Sender<ClientRequest>,
     state: &SharedState,
 ) -> Result<()> {
     let key: Key = event.into();
@@ -114,37 +107,23 @@ fn handle_key_event(
         key_sequence = KeySequence { keys: vec![key] };
     }
 
-    let ui = state.ui.lock();
-    let handled = match ui.popup {
-        None => {
-            // no popup
-            match ui.current_page() {
-                PageState::Library => {
-                    drop(ui);
-                    window::handle_key_sequence_for_library_window(&key_sequence, state)?
-                }
-                PageState::Recommendations(..) => {
-                    drop(ui);
-                    window::handle_key_sequence_for_recommendation_window(
-                        &key_sequence,
-                        send,
-                        state,
-                    )?
-                }
-                PageState::Context(..) => {
-                    drop(ui);
-                    window::handle_key_sequence_for_context_window(&key_sequence, send, state)?
-                }
-                PageState::Searching { .. } => {
-                    drop(ui);
-                    window::handle_key_sequence_for_search_window(&key_sequence, send, state)?
-                }
+    let handled = if state.ui.lock().popup.is_none() {
+        // no popup
+        let page_type = state.ui.lock().current_page().page_type();
+        match page_type {
+            PageType::Library => page::handle_key_sequence_for_library_page(&key_sequence, state)?,
+            PageType::Search => {
+                page::handle_key_sequence_for_search_page(&key_sequence, client_pub, state)?
+            }
+            PageType::Context => {
+                page::handle_key_sequence_for_context_page(&key_sequence, client_pub, state)?
+            }
+            PageType::Tracks => {
+                page::handle_key_sequence_for_tracks_page(&key_sequence, client_pub, state)?
             }
         }
-        Some(_) => {
-            drop(ui);
-            popup::handle_key_sequence_for_popup(&key_sequence, send, state)?
-        }
+    } else {
+        popup::handle_key_sequence_for_popup(&key_sequence, client_pub, state)?
     };
 
     // if the key sequence is not handled, let the global command handler handle it
@@ -153,7 +132,7 @@ fn handle_key_event(
             .keymap_config
             .find_command_from_key_sequence(&key_sequence)
         {
-            Some(command) => handle_global_command(command, send, state)?,
+            Some(command) => handle_global_command(command, client_pub, state)?,
             None => false,
         }
     } else {
@@ -173,7 +152,7 @@ fn handle_key_event(
 /// handles a global command
 fn handle_global_command(
     command: Command,
-    send: &mpsc::Sender<ClientRequest>,
+    client_pub: &mpsc::Sender<ClientRequest>,
     state: &SharedState,
 ) -> Result<bool> {
     let mut ui = state.ui.lock();
@@ -183,25 +162,27 @@ fn handle_global_command(
             ui.is_running = false;
         }
         Command::NextTrack => {
-            send.send(ClientRequest::Player(PlayerRequest::NextTrack))?;
+            client_pub.blocking_send(ClientRequest::Player(PlayerRequest::NextTrack))?;
         }
         Command::PreviousTrack => {
-            send.send(ClientRequest::Player(PlayerRequest::PreviousTrack))?;
+            client_pub.blocking_send(ClientRequest::Player(PlayerRequest::PreviousTrack))?;
         }
         Command::ResumePause => {
-            send.send(ClientRequest::Player(PlayerRequest::ResumePause))?;
+            client_pub.blocking_send(ClientRequest::Player(PlayerRequest::ResumePause))?;
         }
         Command::Repeat => {
-            send.send(ClientRequest::Player(PlayerRequest::Repeat))?;
+            client_pub.blocking_send(ClientRequest::Player(PlayerRequest::Repeat))?;
         }
         Command::Shuffle => {
-            send.send(ClientRequest::Player(PlayerRequest::Shuffle))?;
+            client_pub.blocking_send(ClientRequest::Player(PlayerRequest::Shuffle))?;
         }
         Command::VolumeUp => {
             if let Some(ref playback) = state.player.read().playback {
                 if let Some(percent) = playback.device.volume_percent {
                     let volume = std::cmp::min(percent + 5, 100_u32);
-                    send.send(ClientRequest::Player(PlayerRequest::Volume(volume as u8)))?;
+                    client_pub.blocking_send(ClientRequest::Player(PlayerRequest::Volume(
+                        volume as u8,
+                    )))?;
                 }
             }
         }
@@ -209,7 +190,9 @@ fn handle_global_command(
             if let Some(ref playback) = state.player.read().playback {
                 if let Some(percent) = playback.device.volume_percent {
                     let volume = std::cmp::max(percent.saturating_sub(5_u32), 0_u32);
-                    send.send(ClientRequest::Player(PlayerRequest::Volume(volume as u8)))?;
+                    client_pub.blocking_send(ClientRequest::Player(PlayerRequest::Volume(
+                        volume as u8,
+                    )))?;
                 }
             }
         }
@@ -217,7 +200,7 @@ fn handle_global_command(
             ui.popup = Some(PopupState::CommandHelp { offset: 0 });
         }
         Command::RefreshPlayback => {
-            send.send(ClientRequest::GetCurrentPlayback)?;
+            client_pub.blocking_send(ClientRequest::GetCurrentPlayback)?;
         }
         Command::ShowActionsOnCurrentTrack => {
             if let Some(track) = state.player.read().current_playing_track() {
@@ -226,49 +209,70 @@ fn handle_global_command(
                 }
             }
         }
-        Command::BrowsePlayingContext => {
-            ui.create_new_page(PageState::Context(None, ContextPageType::CurrentPlaying));
+        Command::CurrentlyPlayingContextPage => {
+            ui.create_new_page(PageState::Context {
+                id: None,
+                context_page_type: ContextPageType::CurrentPlaying,
+                state: None,
+            });
         }
         Command::BrowseUserPlaylists => {
-            send.send(ClientRequest::GetUserPlaylists)?;
+            client_pub.blocking_send(ClientRequest::GetUserPlaylists)?;
             ui.popup = Some(PopupState::UserPlaylistList(
                 PlaylistPopupAction::Browse,
                 new_list_state(),
             ));
         }
         Command::BrowseUserFollowedArtists => {
-            send.send(ClientRequest::GetUserFollowedArtists)?;
+            client_pub.blocking_send(ClientRequest::GetUserFollowedArtists)?;
             ui.popup = Some(PopupState::UserFollowedArtistList(new_list_state()));
         }
         Command::BrowseUserSavedAlbums => {
-            send.send(ClientRequest::GetUserSavedAlbums)?;
+            client_pub.blocking_send(ClientRequest::GetUserSavedAlbums)?;
             ui.popup = Some(PopupState::UserSavedAlbumList(new_list_state()));
         }
+        Command::TopTrackPage => {
+            ui.create_new_page(PageState::Tracks {
+                id: "top-tracks".to_string(),
+                title: "Top Tracks".to_string(),
+                desc: "User's top tracks".to_string(),
+                state: new_table_state(),
+            });
+            client_pub.blocking_send(ClientRequest::GetUserTopTracks)?;
+        }
+        Command::RecentlyPlayedTrackPage => {
+            ui.create_new_page(PageState::Tracks {
+                id: "recently-played-tracks".to_string(),
+                title: "Recently Played Tracks".to_string(),
+                desc: "User's recently played tracks".to_string(),
+                state: new_table_state(),
+            });
+            client_pub.blocking_send(ClientRequest::GetUserRecentlyPlayedTracks)?;
+        }
         Command::LibraryPage => {
-            ui.create_new_page(PageState::Library);
+            ui.create_new_page(PageState::Library {
+                state: LibraryPageUIState::new(),
+            });
+            client_pub.blocking_send(ClientRequest::GetUserPlaylists)?;
+            client_pub.blocking_send(ClientRequest::GetUserFollowedArtists)?;
+            client_pub.blocking_send(ClientRequest::GetUserSavedAlbums)?;
         }
         Command::SearchPage => {
-            ui.create_new_page(PageState::Searching {
-                input: "".to_owned(),
-                current_query: "".to_owned(),
+            ui.create_new_page(PageState::Search {
+                input: String::new(),
+                current_query: String::new(),
+                state: SearchPageUIState::new(),
             });
         }
         Command::PreviousPage => {
             if ui.history.len() > 1 {
                 ui.history.pop();
                 ui.popup = None;
-                ui.window = WindowState::Unknown;
-
-                // empty the previous page's `context_id` to force
-                // updating the context page's window state and requesting the context data
-                if let PageState::Context(ref mut context_id, _) = ui.current_page_mut() {
-                    *context_id = None;
-                }
             }
         }
         Command::SwitchDevice => {
             ui.popup = Some(PopupState::DeviceList(new_list_state()));
-            send.send(ClientRequest::GetDevices)?;
+            client_pub.blocking_send(ClientRequest::GetDevices)?;
         }
         Command::SwitchTheme => {
             // get the available themes with the current theme moved to the first position
@@ -281,6 +285,20 @@ fn handle_global_command(
 
             ui.popup = Some(PopupState::ThemeList(themes, new_list_state()));
         }
+        Command::ReconnectIntegratedClient => {
+            #[cfg(feature = "streaming")]
+            client_pub.blocking_send(ClientRequest::NewSpircConnection)?;
+        }
+        Command::FocusNextWindow => {
+            if !ui.has_focused_popup() {
+                ui.current_page_mut().next()
+            }
+        }
+        Command::FocusPreviousWindow => {
+            if !ui.has_focused_popup() {
+                ui.current_page_mut().previous()
+            }
+        }
         _ => return Ok(false),
     }
     Ok(true)
