@@ -2,18 +2,18 @@ mod auth;
 mod client;
 mod command;
 mod config;
-#[cfg(feature = "streaming")]
-mod connect;
 mod event;
 mod key;
+#[cfg(feature = "streaming")]
+mod spirc;
 mod state;
 mod token;
 mod ui;
 mod utils;
 
 fn init_app_cli_arguments() -> clap::ArgMatches {
-    clap::App::new("spotify-player")
-        .version("0.5.2")
+    clap::Command::new("spotify-player")
+        .version("0.6.0")
         .about("A command driven spotify player")
         .author("Thang Pham <phamducthang1234@gmail>")
         .arg(
@@ -42,12 +42,54 @@ fn init_app_cli_arguments() -> clap::ArgMatches {
         .get_matches()
 }
 
+async fn init_spotify(
+    client_pub: &tokio::sync::mpsc::Sender<event::ClientRequest>,
+    spirc_pub: &tokio::sync::broadcast::Sender<()>,
+    client: &client::Client,
+    state: &state::SharedState,
+) -> anyhow::Result<()> {
+    client.init_token().await?;
+    client.update_current_playback_state(state).await?;
+
+    // if `streaming` feature is enabled, create new Spirc connection
+    #[cfg(feature = "streaming")]
+    client
+        .new_spirc_connection(spirc_pub.subscribe(), client_pub.clone(), false)
+        .await?;
+
+    if state.player.read().playback.is_none() {
+        tracing::info!(
+            "no playback found on startup, trying to connect to the first available device..."
+        );
+        client.connect_to_first_available_device().await?;
+    }
+
+    // Request user data
+
+    client_pub
+        .send(event::ClientRequest::GetCurrentUser)
+        .await?;
+
+    // request data needed to render the Library page (default page when starting the application)
+    client_pub
+        .send(event::ClientRequest::GetUserPlaylists)
+        .await?;
+    client_pub
+        .send(event::ClientRequest::GetUserFollowedArtists)
+        .await?;
+    client_pub
+        .send(event::ClientRequest::GetUserSavedAlbums)
+        .await?;
+
+    Ok(())
+}
+
 #[tokio::main]
 async fn main() -> anyhow::Result<()> {
     // parse command line arguments
     let args = init_app_cli_arguments();
 
-    // initialize the application's cache folders and config folder
+    // initialize the application's cache folder and config folder
     let config_folder = match args.value_of("config-folder") {
         Some(path) => path.into(),
         None => config::get_config_folder_path()?,
@@ -56,7 +98,6 @@ async fn main() -> anyhow::Result<()> {
         Some(path) => path.into(),
         None => config::get_cache_folder_path()?,
     };
-
     if !config_folder.exists() {
         std::fs::create_dir_all(&config_folder)?;
     }
@@ -82,55 +123,55 @@ async fn main() -> anyhow::Result<()> {
     state.parse_config_files(&config_folder, args.value_of("theme"))?;
     let state = std::sync::Arc::new(state);
 
-    // initialize a librespot session
+    // create a librespot session
     let session = auth::new_session(&cache_folder, state.app_config.device.audio_cache).await?;
 
-    // start application's threads
-    let (send, recv) = std::sync::mpsc::channel::<event::ClientRequest>();
+    // create a spotify API client
+    let client = client::Client::new(
+        session.clone(),
+        state.app_config.device.clone(),
+        state.app_config.client_id.clone(),
+    );
 
-    // get some prior information
-    send.send(event::ClientRequest::GetCurrentUser)?;
-    send.send(event::ClientRequest::GetCurrentPlayback)?;
+    // create application's channels
+    let (client_pub, client_sub) = tokio::sync::mpsc::channel::<event::ClientRequest>(16);
+    let (spirc_pub, _) = tokio::sync::broadcast::channel::<()>(16);
 
-    // connection thread (used to initialize the integrated Spotify client using librespot)
-    #[cfg(feature = "streaming")]
-    std::thread::spawn({
-        let session = session.clone();
-        let send = send.clone();
-        let device = state.app_config.device.clone();
-        move || {
-            connect::new_connection(session, send, device);
-        }
-    });
+    // initialize Spotify-related stuff
+    init_spotify(&client_pub, &spirc_pub, &client, &state).await?;
 
-    // client event handler thread
-    std::thread::spawn({
+    // Spawn application's tasks
+
+    // client event handler task
+    tokio::task::spawn({
         let state = state.clone();
-        let client = client::Client::new(session, state.app_config.client_id.clone());
-        client.init_token().await?;
-        move || {
-            client::start_client_handler(state, client, recv);
+        let client_pub = client_pub.clone();
+        async move {
+            client::start_client_handler(state, client, client_pub, client_sub, spirc_pub).await;
         }
     });
 
-    // terminal event handler thread
-    std::thread::spawn({
-        let send = send.clone();
+    // terminal event handler task
+    tokio::task::spawn_blocking({
+        let client_pub = client_pub.clone();
         let state = state.clone();
         move || {
-            event::start_event_handler(send, state);
+            event::start_event_handler(state, client_pub);
         }
     });
 
-    // player event watcher thread(s)
-    std::thread::spawn({
-        let send = send.clone();
+    // player event watcher task
+    tokio::task::spawn_blocking({
         let state = state.clone();
         move || {
-            client::start_player_event_watchers(state, send);
+            client::start_player_event_watchers(state, client_pub);
         }
     });
 
-    // application's UI as the main thread
-    ui::start_ui(state, send)
+    // application's UI as the main task
+    tokio::task::spawn_blocking(move || {
+        ui::start_ui(state).unwrap();
+    })
+    .await?;
+    std::process::exit(0);
 }
