@@ -1,5 +1,6 @@
 pub use rspotify::model as rspotify_model;
 pub use rspotify::model::{AlbumId, ArtistId, Id, PlaylistId, TrackId, UserId};
+use serde::{Deserialize, Serialize};
 
 use crate::command;
 
@@ -128,6 +129,28 @@ pub struct Playlist {
     pub owner: (String, UserId),
 }
 
+// NOTE: the below cursor-based paging objects is a workaround to account
+// the fact that `rspotify v0.11.3` assumes `cursors` field in the response is not nullable.
+
+/// Cursor-based paging object
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Eq, Default)]
+pub struct CursorBasedPage<T> {
+    pub href: String,
+    pub items: Vec<T>,
+    pub limit: u32,
+    pub next: Option<String>,
+    pub cursors: Option<Cursor>,
+    /// Absent if it has read all data items. This field doesn't match what
+    /// Spotify document says
+    pub total: Option<u32>,
+}
+
+/// Cursor object
+#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Eq, Default)]
+pub struct Cursor {
+    pub after: Option<String>,
+}
+
 impl Context {
     /// sorts tracks in the context by a sort oder
     pub fn sort_tracks(&mut self, sort_order: TrackOrder) {
@@ -238,6 +261,16 @@ impl Item {
             Self::Playlist(_) => vec![command::Action::SaveToLibrary],
         }
     }
+
+    /// gets the name of the spotify item
+    pub fn name(&self) -> &str {
+        match self {
+            Self::Track(track) => &track.name,
+            Self::Artist(artist) => &artist.name,
+            Self::Album(album) => &album.name,
+            Self::Playlist(playlist) => &playlist.name,
+        }
+    }
 }
 
 impl SeedItem {
@@ -385,8 +418,7 @@ fn from_simplified_artists_to_artists(
 ) -> Vec<Artist> {
     artists
         .into_iter()
-        .map(Artist::try_from_simplified_artist)
-        .flatten()
+        .filter_map(Artist::try_from_simplified_artist)
         .collect()
 }
 
